from setuptools import setup, find_packages
from setuptools.dist import get_metadata_version


classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Education',
    'Operating System :: OS Independent',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3' 
]

with open("README.txt", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='pentesterbasicbankaccount',
    version='0.0.2',
    description='Allows you to create a simple bank account',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='',
    author='pentester56',
    author_email='helloworld@gmail.com',
    license='MIT',
    classifiers=classifiers,
    keywords='',
    entry_points = {
        'console_scripts': ['rexona=account.app:main'],
    },
    packages=find_packages(),
    install_requires=['argparse']

)